@@ -1917,7 +1917,7 @@ f_writefile(typval_T *argvars, typval_T *rettv)
 	list = argvars[0].vval.v_list;
 	if (list == NULL)
 	    return;
-	range_list_materialize(list);
+	CHECK_LIST_MATERIALIZE(list);
 	FOR_ALL_LIST_ITEMS(list, li)
 	    if (tv_get_string_chk(&li->li_tv) == NULL)
 		return;
